<?php

namespace App\Command;

use App\Entity\Users;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

#[AsCommand(
    name: 'app:make:user',
    description: 'Add a short description for your command',
)]
class CreateUserCommand extends Command
{
    public function __construct(
        private ManagerRegistry $doctrine,
        private UserPasswordHasherInterface $passwordHasher,
        string $name = null
    ) {
        parent::__construct($name);
    }

    protected function configure(): void
    {
        $this
            ->addOption('username', 'u', InputOption::VALUE_OPTIONAL, 'Username of the new user')
            ->addOption('password', 'p', InputOption::VALUE_OPTIONAL, 'Password for the new user')
            ->addOption('name', 'd', InputOption::VALUE_NONE, 'Display name of the user')
            ->addOption('admin', 'a', InputOption::VALUE_NONE, 'Create a admin user')
            ->addOption('noadmin', 'na', InputOption::VALUE_NONE, 'Create a normal user')
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);

        $user = new Users();
        if ($input->getOption('username')) {
            $user->setUsername($input->getOption('username'));
        } else {
            $user->setUsername($io->ask('Please enter a username'));
        }

        if ($input->getOption('password')) {
            $user->setPassword(
                $this->passwordHasher->hashPassword($user, $input->getOption('password'))
            );
        } else {
            $user->setPassword(
                $this->passwordHasher->hashPassword($user, $io->ask('Please enter a password'))
            );
        }

        if ($input->getOption('name')) {
            $user->setName($input->getOption('name'));
        } else {
            $user->setName($io->ask('Please enter a display name for the user'));
        }

        if ($input->getOption('admin')) {
            $user->setAdminAccess(true);
        } elseif (!$input->getOption('noadmin')) {
            $user->setAdminAccess($io->confirm('Is the new user a admin user', false));
        }

        if ($this->doctrine->getRepository(Users::class)->findOneBy(['username' => $user->getUsername()])) {
            $io->error("Can't create two users with same username");
            return Command::FAILURE;
        }

        $entityManager = $this->doctrine->getManager();
        $entityManager->persist($user);
        $entityManager->flush();



        return Command::SUCCESS;
    }
}
