<?php

namespace App\Controller;

use App\Entity\Users;
use Doctrine\Persistence\ManagerRegistry;
use Scheb\TwoFactorBundle\Security\TwoFactor\Provider\Google\GoogleAuthenticatorInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\ConflictHttpException;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Annotation\Route;

class ProfileController extends AbstractController
{
    #[Route('/profile', name: 'app_profile')]
    public function index(): Response
    {
        /** @var Users $username */
        $username = $this->getUser();
        return $this->render('pages/profile.html.twig', [
            'name' => $username->getName()
        ]);
    }

    #[Route('/profile/update', name: 'app_profile_update', methods: ['POST'])]
    public function update(Request $request, ManagerRegistry $doctrine): Response
    {
        /**
         * @var Users $user
         */
        $user = $this->getUser();
        $user->setName($request->request->getString('inputName'));

        $doctrine->getManager()->flush();

        return $this->redirectToRoute('app_profile');
    }

    #[Route('/profile/password', name:'app_profile_password_change', methods: ['POST'])]
    public function changePassword(
        Request $request,
        ManagerRegistry $doctrine,
        UserPasswordHasherInterface $passwordHasher
    ): Response {
        if ($request->request->get('inputPassword') !== $request->request->get('inputPasswordConfirm')) {
            $this->addFlash('error', [
                'title' => 'Passwort not changed',
                'message' => 'Password confirmation doesn\'t match, please try again.'
            ]);
            return $this->redirectToRoute('app_profile');
        }

        /** @var Users $user */
        $user = $this->getUser();
        $user->setPassword(
            $passwordHasher->hashPassword($user, $request->request->getString('inputPassword'))
        );
        $doctrine->getManager()->flush();

        $this->addFlash('success', [
            'title' => 'Passwort changed',
            'message' => 'Your password was changed.'
        ]);
        return $this->redirectToRoute('app_profile');
    }

    #[Route('/profile/totpEnable', name:'app_profile_totp_enable', methods:['POST'])]
    public function enableTotp(): Response
    {
        return $this->redirectToRoute('app_profile');
    }

    #[Route('/profile/totp/config', methods: ['GET'])]
    public function totpConfig(GoogleAuthenticatorInterface $totpInterface, ManagerRegistry $doctrine): Response
    {
        /**
         * @var Users $user
         */
        $user = $this->getUser();
        $secret = $totpInterface->generateSecret();
        $user->setGoogleAuthenticatorSecret($secret);
        $doctrine->getManager()->flush();

        return new JsonResponse(
            [
                'secret' => $secret,
                'qrData' => $totpInterface->getQRContent($user),
            ]
        );
    }

    #[Route('/profile/totp/enable', methods: ['POST'], name: 'app_profile_totp_enable')]
    public function totpEnable(
        GoogleAuthenticatorInterface $totpInterface,
        ManagerRegistry $doctrine,
        Request $request
    ): Response {
        $code = $request->request->getString('code');
        /**
         * @var Users $user
         */
        $user = $this->getUser();
        if (!$totpInterface->checkCode($user, $code)) {
            $this->addFlash('error', [
                'title' => 'TOTP not enabled',
                'message' => 'The code you entered is invalid, please try again.'
            ]);
            return $this->redirectToRoute('app_profile');
        }
        $user->setGoogleAuthenticatorEnabled(true);
        $doctrine->getManager()->flush();
        return $this->redirectToRoute('app_profile');
    }

    #[Route('/profile/totp/disable', methods: ['POST'], name: 'app_profile_totp_disable')]
    public function totpDisable(ManagerRegistry $doctrine): Response
    {
        $this->addFlash('info', [
            'title' => 'TOTP Disabled',
            'message' => 'TOTP was disabled successfully'
        ]);
        /** @var Users $user */
        $user = $this->getUser();
        $user->setGoogleAuthenticatorEnabled(false);
        $doctrine->getManager()->flush();
        return $this->redirectToRoute('app_profile');
    }
}
