<?php

namespace App\Entity;

use App\Repository\UsersRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Scheb\TwoFactorBundle\Model\Google\TwoFactorInterface;
use Symfony\Component\Security\Core\User\PasswordAuthenticatedUserInterface;
use Symfony\Component\Security\Core\User\UserInterface;

#[ORM\Entity(repositoryClass: UsersRepository::class)]
class Users implements UserInterface, PasswordAuthenticatedUserInterface, TwoFactorInterface
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column]
    private ?int $id = null;

    #[ORM\Column(length: 255, unique: true)]
    private ?string $username = null;

    #[ORM\Column(length: 255)]
    private ?string $password = null;

    #[ORM\Column]
    private ?bool $adminAccess = false;

    #[ORM\ManyToMany(targetEntity: WebResetter::class, inversedBy: 'users')]
    private Collection $webResetter;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $name = null;

    #[ORM\OneToMany(mappedBy: 'user', targetEntity: History::class)]
    private Collection $histroy;

    #[ORM\Column]
    private ?bool $enabled = true;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $totpSecret = null;

    #[ORM\Column]
    private ?bool $twoFactorEnabled = false;

    public function __construct()
    {
        $this->webResetter = new ArrayCollection();
        $this->histroy = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getUsername(): ?string
    {
        return $this->username;
    }

    public function setUsername(string $username): self
    {
        $this->username = $username;

        return $this;
    }

    public function getPassword(): ?string
    {
        return $this->password;
    }

    public function setPassword(string $password): self
    {
        $this->password = $password;

        return $this;
    }

    public function isAdminAccess(): ?bool
    {
        return $this->adminAccess;
    }

    public function setAdminAccess(bool $adminAccess): self
    {
        $this->adminAccess = $adminAccess;

        return $this;
    }

    public function getRoles(): array
    {
        return array_unique([
            'ROLE_USER',
            $this->isAdminAccess() ? 'ROLE_ADMIN' : null
        ]);
    }

    public function eraseCredentials()
    {
    }

    public function getUserIdentifier(): string
    {
        return $this->username;
    }

    /**
     * @return Collection<int, WebResetter>
     */
    public function getWebResetter(): Collection
    {
        return $this->webResetter;
    }

    public function addWebResetter(WebResetter $webResetter): self
    {
        if (!$this->webResetter->contains($webResetter)) {
            $this->webResetter->add($webResetter);
        }

        return $this;
    }

    public function removeWebResetter(WebResetter $webResetter): self
    {
        $this->webResetter->removeElement($webResetter);

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    /**
     * @return Collection<int, History>
     */
    public function getHistroy(): Collection
    {
        return $this->histroy;
    }

    public function addHistroy(History $histroy): self
    {
        if (!$this->histroy->contains($histroy)) {
            $this->histroy->add($histroy);
            $histroy->setUser($this);
        }

        return $this;
    }

    public function removeHistroy(History $histroy): self
    {
        if ($this->histroy->removeElement($histroy)) {
            // set the owning side to null (unless already changed)
            if ($histroy->getUser() === $this) {
                $histroy->setUser(null);
            }
        }

        return $this;
    }

    public function isEnabled(): ?bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): self
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function isGoogleAuthenticatorEnabled(): bool
    {
        return $this->twoFactorEnabled;
    }

    public function getGoogleAuthenticatorUsername(): string
    {
        return $this->username;
    }

    public function getGoogleAuthenticatorSecret(): ?string
    {
        return $this->totpSecret;
    }

    public function setGoogleAuthenticatorSecret(?string $totpSecret): self
    {
        $this->totpSecret = $totpSecret;

        return $this;
    }

    public function setGoogleAuthenticatorEnabled(bool $enabled): self
    {
        $this->twoFactorEnabled = $enabled;

        return $this;
    }
}
